#!/usr/bin/ipstcl2

# VERSION: 1.20170608
# COPYRIGHT(c) 2017 CryptoPhoto -- http://cryptophoto.com/
# AUTHOR: CryptoPhoto
#
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

package require cryptophotoutils

source cryptophotoconf.tcl

source debug.tcl
variable dFile "/tmp/cryptophotosettings.debug"

namespace eval ::cryptophotosettings {
    global method
    global pubkey
    global privkey
    global salt
    global server

    # Main starts here.
    if {[info exists env(REMOTE_USER)]} {
	set cur_user $env(REMOTE_USER)
    } else {
	set cur_user "who_am_i"
    }

    set ip "127.0.0.1"
    if {[info exists env(REMOTE_ADDR)]} {
        set ip $env(REMOTE_ADDR)
    }

    set conf [dict create pubkey $pubkey privkey $privkey salt $salt server $server uid $cur_user]
    cryptophotoutils::reset_vars $conf

    set resp [cryptophotoutils::start_session $ip "false"]

    set sid ""
    set err ""
    set errIP ""
    if {[dict get $resp isValid] == 1} {
	set sid [dict get $resp sid]
    } else {
	set err [dict get $resp err]
	if { [dict exists $resp errIP] } {
	    set errIP [dict get $resp errIP]
	}
    }

    # ipso -nohtml
    set method [get_Method]
    MIME text

    source json.tcl
    set outputDict [dict create success true]

    if { $err ne "" } {
	dict set outputDict success false
    }

    set dataDict [dict create cur_user $cur_user ]

    if { $err ne "" } {
	dict set dataDict err $err
        if { $errIP ne "" } {
	    dict set dataDict errIP $errIP
        }
    }

    if { $sid ne "" } {
	dict set dataDict sid $sid
	dict set dataDict server $server
    }

    dict set outputDict data $dataDict
    HTML [::dict2json::compile_json {dict data {dict} } $outputDict]

}; # namespace
exit 0
