#!/usr/bin/ipstcl2

# VERSION: 1.20170613
# COPYRIGHT(c) 2017 CryptoPhoto -- http://cryptophoto.com/
# AUTHOR: CryptoPhoto
#
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

package require cryptophotoutils
source cryptophotoconf.tcl

source debug.tcl
variable dFile "/tmp/login_adv.debug"

proc saveLockFile {filename arr} {
    array set myarray $arr

    set file [open $filename w]
    puts $file [array get myarray]
    close $file
}

variable form
proc collect_form {} {
    global form env
    if [info exists env(CONTENT_LENGTH)] {
        set forminput [read stdin $env(CONTENT_LENGTH)]
        set form(method) POST
    } else {
        set forminput $env(QUERY_STRING)
        set form(method) GET
    }
    foreach {name value} [split $forminput =&] {
        regsub -all \\\+ $value " " value
        regsub -all -nocase {%([0-9a-f][0-9a-f])} $value \
                {[format %c 0x\1]} value
        set form($name) [subst $value]
    }
}

proc redirect {url} {

    MIME html

    HTML "<!DOCTYPE html>"
    HTML "<html><head>"
    HTML "<script type=\"text/javascript\">"
    HTML "location.href = \"$url\";"
    HTML "</script></head><body></body>"
    HTML "</html>"
    exit 0
}

proc post_redirect {url userName userPass} {
    set data(uname) $userName
    saveLockFile "/tmp/_cplock_$userName" [array get data]

    MIME html

    HTML "<!DOCTYPE html>"
    HTML "<html><head></head>"
    HTML "<body>"
    HTML "<form name='redirectForm' action='$url' method='POST'>"
    HTML "<input type='hidden' name='userName' value='$userName' />"
    HTML "<input type='hidden' name='userPass' value='$userPass' />"
    HTML "<input type='submit' value='Redirect' style='display:none' />"
    HTML "</form><script>document.forms\['redirectForm'\].submit()</script></html>"
    exit 0
}

if {[info exists env(MULTIPORTAL_PATH_PREFIX)]} {

    set MP_PREFIX1  $env(MULTIPORTAL_PATH_PREFIX)
    set MP_PREFIX_LEN [string last  "/" $MP_PREFIX1]

    if { ${MP_PREFIX_LEN} < 0 } {
        set MP_PREFIX ""
    } else {
        set MP_PREFIX [string range $MP_PREFIX1 1 $MP_PREFIX_LEN]
    }
} else {
    set MP_PREFIX ""
}

if {$MP_PREFIX ne ""} {
    set MP_PREFIX "/$MP_PREFIX"
}

set method ""
if {[info exists env(REQUEST_METHOD)]} {
    set method $env(REQUEST_METHOD)
}

set ip "127.0.0.1"
if {[info exists env(REMOTE_ADDR)]} {
    set ip $env(REMOTE_ADDR)
}

if {[info exists env(CONTENT_LENGTH)]} {
    # collect POST/GET data
    collect_form
}

set cur_user ""
set cur_pass ""

if {[info exists form(userName)] && [info exists form(userPass)]} {
    set cur_user $form(userName)
    set cur_pass $form(userPass)
}

if { $cpenabled == 0 } {
    redirect "$MP_PREFIX/cgi-bin/login.tcl"
}

if {($cur_user eq "") || ($cur_pass eq "")} {
    redirect "$MP_PREFIX/cgi-bin/login.tcl"
}

if { ($cur_user ne "")&& ([file exists "/tmp/_cplock_$cur_user"] == 1)} {
    file delete "/tmp/_cplock_$cur_user"
}

set conf [dict create pubkey $pubkey privkey $privkey salt $salt server $server uid $cur_user]
cryptophotoutils::reset_vars $conf

set cpsid ""
set err ""
set errIP ""

if {[info exists env(CONTENT_LENGTH)]} {

    set cplogin 0
    catch { set cplogin $form(cplogin) } e
    if { $cplogin eq "1" } {
        set cp_phc ""
        set selector ""
        set col ""
        set row ""

        catch { set cp_phc $form(cp_phc) } e
        catch { set selector $form(token_selector) } e
        catch { set col $form(token_response_field_col) } e
        catch { set row $form(token_response_field_row) } e

        if { ($selector ne "") && ( ($cp_phc ne "") || ( $col ne "" || $row ne "" )  ) } {

            set r [cryptophotoutils::verify_response $selector $row $col $cp_phc $ip]

            if {[dict get $r isValid] == 1} {
                post_redirect "$MP_PREFIX/cgi-bin/home.tcl" $form(userName) $form(userPass)
        } else {
                set err "Codes aren't valid"
            }
        }
    }
}

set resp [cryptophotoutils::start_session $ip "true"]

if {[dict get $resp isValid] == 1} {
    if {[dict get $resp hasToken] eq "true"} {
        set cpsid [dict get $resp sid]
    } else {
        post_redirect "$MP_PREFIX/cgi-bin/home.tcl" $cur_user $cur_pass
    }
} else {
    set err [dict get $resp err]
    if { [dict exists $resp errIP] } {
        set errIP [dict get $resp errIP]
    }
}

MIME html

HTML "<!doctype html>"
HTML "<HTML><HEAD>\n"
HTML "<meta name=\"others\" content=\"WEBUI CRYPTOPHOTO AUTHENTICATION PAGE\"  />"
HTML "<TITLE>CryptoPhoto Authentication</TITLE>\n"

# note: styles in-line because styles are not used when loaded from CSS
HTML "<link rel=\"stylesheet\" type=\"text/css\" href=\"/login/cryptophoto/ext-all.css\" />\n"
HTML "<link rel=\"stylesheet\" type=\"text/css\" href=\"/login/cryptophoto/cp.css\" />\n"

# IE has a different background color
HTML "<STYLE TYPE=\"text/css\">\n"
HTML ".ext-ie .webui-login-fld{font-size: 11px;}"
HTML "\n</STYLE>\n"

HTML "<script type=\"text/javascript\" src=\"/login/cryptophoto/ext-base.js\"></script>"
HTML "<script type=\"text/javascript\" src=\"/login/cryptophoto/ext-all.js\"></script>"
HTML "<script type=\"text/javascript\">"
HTML "var errMsgText = \"$err\";"
HTML "var bannerMsgText = \"\";"
HTML "var cpserver = \"$server\";"
if { $cpsid ne "" } {
    HTML "var cpsid = \"$cpsid\";"
} else {
    HTML "var cpsid = \"\";"
}
HTML "bannerMsgText += \"CryptoPhoto Authentication\";"
HTML "var hostname=\'\';"
HTML "var version=\'&nbsp;\';"
HTML "var formAction=\"/cgi-bin/login_adv.tcl\";"
HTML "var uname=\"${cur_user}\";"
HTML "var upass=\"${cur_pass}\";"
HTML "</script>"
HTML "<script type=\"text/javascript\" src=\"/login/cryptophoto/cryptophoto.js\"></script>"
HTML "</HEAD>"
HTML "<BODY></BODY></HTML>"
